/*
 * Implementation eines HTTP Servers
 */

#ifdef _REENTRANT
#include <pthread.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <socket_class.h>
#include "HttpConfig.h"
#include "MimeTypes.h"
#include "fpSocket.h"
#include "misc.h"
#include "HttpServer.h"
#include "HttpRequest.h"

/*
 * ----------------------------------------------------------------------
 * Server
 * ----------------------------------------------------------------------
 */

const char * HttpServer::ServerName  = "fpServ/1.0";
int          HttpServer::logrequests = 0;

HttpServer::HttpServer (const char * config, FILE * l)
  : log(l)
{
  char temppath[1024];

#ifdef _REENTRANT
  pthread_mutex_init (&mutex, NULL);
#endif

  if (config) {
    PrintLog  ("loading config %s", config);
    conf.load (config);
  }

  if (conf["MimeTypes"] != NULL) {
    PrintLog  ("loading mime types from %s", conf["MimeTypes"]);
    mime.load (conf["MimeTypes"]);
  }

  if (conf["TransferLog"] != NULL) {
    sprintf (temppath, "%s%s%s",
	     (conf["TransferLog"][0] == '/') ? "" : (conf["ServerRoot"]?conf["ServerRoot"]:"."),
	     (conf["TransferLog"][0] == '/') ? "" : "/",
	     conf["TransferLog"]);
    if ((transfer_log = fopen (temppath, "a")) == NULL) {
      PrintLog ("warning: could not open transfer log %s", temppath);
    } else {
      PrintLog ("using transfer log %s", temppath);
    }
  } else {
    transfer_log = NULL;
  }

  portno = 80;
}

HttpServer::~HttpServer (void)
{
  fclose (transfer_log);
#ifdef _REENTRANT
  pthread_mutex_destroy (&mutex);
#endif
}

void
HttpServer::PrintLog (const char * fmt, ...)
{
  va_list args;
  va_start(args, fmt);

  if (log) {
#ifdef _REENTRANT
    pthread_mutex_lock (&mutex);
#endif
    fprintf  (log, "HttpServer:  ");
    vfprintf (log, fmt, args);
    fputc    ('\n', log);
    fflush   (log);
#ifdef _REENTRANT
    pthread_mutex_unlock (&mutex);
#endif
  }

  va_end (args);
}

#ifdef _REENTRANT

/*
 * Wir brauchen eine statische Funktion fuer pthread_create
 */

void *
HttpServer::service (void * ptr)
{
  blubber * st = (blubber *) ptr;
  char quark[42];

  HttpRequest nreq(st->conn, st->master);
  nreq.serve();

#if 0
  st->conn->fgets (quark, 42);
#endif

  delete st->conn;
  delete st;
  
  return NULL;
}

#endif

int
HttpServer::serve (u_short p)
{
  char hostname[128];

  if (p) {
    portno = p;
  } else if (conf["Port"]) {
    portno = atoi (conf["Port"]);
  }

  PrintLog ("listening on port %d", (int) portno);

  c_address port(portno);
  fpSocket sock(port);

  if (!sock) {
    PrintLog ("error creating socket: %s", sock.whatswrong());
    return -1;
  }

#ifdef _REENTRANT
  pthread_attr_t tha;
  pthread_attr_init (&tha);
  pthread_attr_setdetachstate (&tha, PTHREAD_CREATE_DETACHED);
#endif

  while (42) {
    PrintLog ("waiting for connection");
    fpSocket * conn = sock.accept ();
    if (!*conn) {
      PrintLog ("bogus connection, bailing out");
      break;
    }

    PrintLog ("new connection, peer %s:%d",
	      conn->gethostname_r (hostname, 128),
	      (int) conn->getpeeraddr()->getport());

    /*
     * dup, damit die Variable nicht gleich ueberschrieben wird
     */

#ifdef _REENTRANT
    blubber * st = new blubber;

    if (st != NULL) {
      pthread_t th;

      st->conn   = conn;
      st->master = this;

      if (pthread_create (&th, &tha, service, st) == 0) {
	PrintLog ("made new thread");
	continue;
      }
    }
    PrintLog ("not making new thread");
#endif

    HttpRequest nreq(conn, this);
    nreq.serve();
    delete conn;
  }

  PrintLog ("exiting");
  return 0;
}
