/*
 * Verwalte MimeTypes aus ~/.mime.types
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "MimeTypes.h"
#include "misc.h"


MimeTypes::MimeTypes (void)
{
  types = NULL;
}

MimeTypes::~MimeTypes (void)
{
  _myll * t;

  while (t = types) {
    types = types->next;
    free ((char *) t->type);
    free ((char *) t->ext);
    free (t);
  }
}

void
MimeTypes::put (const char * type, const char * ext)
{
  _myll * t = new _myll;

  if (!t) return;

  t->type = strdup (type);
  t->ext  = strdup (ext);
  t->next = types;
  types   = t;
}

void
MimeTypes::load (const char * file)
{
  char blubber[1024];
  char * type, * ext;
  FILE * f;

  if ((f = fopen (file, "r")) == NULL) {
    return;
  }

  while (!feof (f)) {
    if (fgets (blubber, 1024, f) == NULL)
      break;

    if (blubber[0] == '\0' || blubber[0] == '#' || isspace(blubber[0]))
      continue;

    if ((type = strtok (blubber, " \t\r\n")) == NULL) {
      continue;
    }

    if ((type = strdup (type)) == NULL) {
      continue;
    }

    while ((ext = strtok (NULL, " \t\r\n")) != NULL) {
      put (type, ext);
    }

    free (type);
  }

  fclose (f);
}

const char *
MimeTypes::operator[] (const char * str)
{
  _myll * t = types;

  while (t) {
    if (t->ext && fpstrnicmp (str, t->ext, strlen (str)) == 0 &&
	strlen (str) == strlen (t->ext))
      return t->type;
    t = t->next;
  }
  return NULL;
}
