/*
 * Eine etwas erweiterte Socket-Klasse
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <socket_class.h>
#include <address_class.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#ifdef _REENTRANT
#include <pthread.h>
#endif
#include "fpSocket.h"

#ifdef _REENTRANT
pthread_mutex_t fpSocket::mutex = PTHREAD_MUTEX_INITIALIZER;
#endif


fpSocket::fpSocket (void)
{
  ungot = 0;
}

fpSocket::fpSocket (c_address & addr)
  : c_socket_str (addr)
{
  ungot = 0;
}

fpSocket::fpSocket (c_socket_str & str)
  : c_socket_str (str)
{
  ungot = 0;
}

fpSocket::fpSocket (fpSocket & other)
  : c_socket_str (other)
{
  ungot = other.ungot;
}

fpSocket *
fpSocket::accept (void)
{
  c_socket_str * conn = c_socket_str::accept();
  fpSocket * fconn = new fpSocket(*conn);
  delete conn;
  return fconn;
}

int
fpSocket::fgetc (void)
{
  if (!isconnected()) return EOF;
  if (ungot) {
    ungot = 0;
    return ungotc;
  }
  if (c_socket_str::read (&ungotc, 1) != 1) {
    return EOF;
  }
  return ungotc;
}

int
fpSocket::ungetc (int c)
{
  ungot  = 1;
  ungotc = c;
  return 0;
}

int
fpSocket::read (void * vp, int count)
{
  char * ptr = (char *) vp;
  int res;

  if (count == 0) return  0;
  if (ptr   == 0) return -1;

  if (ungot) {
    ungot = 0;
    *ptr++ = ungotc;
    count--;

    res = c_socket_str::read (ptr, count);

    if (res == -1) {
      return -1;
    }
    return res + 1;
  }

  return c_socket_str::read (ptr, count);
}

char *
fpSocket::fgets (char * ptr, int count)
{
  char * buf = ptr;
  int tc;

  if (!isconnected()) return NULL;

  count--;
  while (count) {
    if ((tc = fgetc ()) == EOF)
      break;
    *ptr = tc;
    if (*ptr == '\x0a') {
      *ptr = '\0';
      return buf;
    } else if (*ptr == '\x0d') {
      if ((*(ptr+1) = fgetc ()) != '\x0a') {
	ungetc (*(ptr+1));
      }
      *ptr = '\0';
      return buf;
    }
    ptr++;
    count--;
  }
  *ptr = '\0';
  return buf;
}

int
fpSocket::printf (const char * fmt, ...)
{
  char blubber[1024];

  va_list args;
  va_start(args, fmt);
  vsprintf(blubber, fmt, args);
  va_end  (args);

  return write (blubber, strlen(blubber));
}

/*
 * Reentrante Funktionen
 */

char *
fpSocket::gethostname_r (char * buf, size_t blen) const
{
  const char * res;
  char * ret;

#ifdef _REENTRANT
  pthread_mutex_lock (&mutex);
#endif

  if ((res = getpeeraddr()->gethostname()) != NULL) {
    strncpy (buf, res, blen);
    if (blen) {
      buf[blen-1] = '\0';
    }
    ret = buf;
  } else {
    ret = NULL;
  }

#ifdef _REENTRANT
  pthread_mutex_unlock (&mutex);
#endif

  return ret;
}

char *
fpSocket::getinetaddr_r (char * buf, size_t blen) const
{
  const char * res;
  char * ret;

#ifdef _REENTRANT
  pthread_mutex_lock (&mutex);
#endif

  if ((res = getpeeraddr()->getinetaddr()) != NULL) {
    strncpy (buf, res, blen);
    if (blen) {
      buf[blen-1] = '\0';
    }
    ret = buf;
  } else {
    ret = NULL;
  }

#ifdef _REENTRANT
  pthread_mutex_unlock (&mutex);
#endif

  return ret;
}

u_long
fpSocket::getnetaddr_r (void) const
{
  u_long res;

#ifdef _REENTRANT
  pthread_mutex_lock (&mutex);
#endif

  res = getpeeraddr()->getnetaddr();

#ifdef _REENTRANT
  pthread_mutex_unlock (&mutex);
#endif

  return res;
}

