
/*
 * Ein paar Funktionen, die nirgendwo sonst hinpassen
 */

#ifdef _REENTRANT
#include <pthread.h>
#endif

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include "misc.h"


int
fpstrnicmp (const char *s1, const char *s2, int len)
{
  if (s1 == NULL && s2 == NULL) return 0;
  if (s1 == NULL) return  1;
  if (s2 == NULL) return -1;

  while (len) {
    if (tolower(*s1) != tolower(*s2)) {
      return (tolower(*s1) - tolower(*s2));
    }
    s1++, s2++, len--;
  }
  return 0;
}

int
fpaddenv (char ***env, const char *s1, const char *s2)
{
  char **iter = *env, **nenv, **niter, *blubb;
  int count=0;

  if ((blubb = (char *) malloc (strlen(s1)+(s2?strlen(s2):0)+2)) == NULL) {
    return -1;
  }

  sprintf (blubb, "%s=%s", s1, (s2)?s2:"");

  if (iter) {
    while (*iter) {
      if (strncmp (*iter, s1, strlen (s1)) == 0) {
	free (*iter);
	*iter = blubb;
	return 0;
      }
      count++;
      iter++;
    }
  }

  if ((nenv = (char **) malloc ((count+2)*sizeof(char*))) == NULL) {
    free (blubb);
    return -1;
  }

  iter  = *env;
  niter = nenv;

  if (iter) {
    while (*iter) {
      *niter = *iter;
      niter++;
      iter++;
    }
  }

  *niter++ = blubb;
  *niter   = NULL;

  free (*env);
  *env = nenv;
  return 0;
}

char *
HttpUnEncode (const char * str)
{
  char *ptr, *result;

  if ((ptr = result = (char *) malloc (strlen(str)+1)) == NULL) {
    return NULL;
  }

  while (*str) {
    if (*str == '+') {
      str++; *ptr++=' ';
    }
    else if (*str == '%' && isxdigit (*(str+1)) && isxdigit (*(str+2))) {
      str++;
      *ptr    = ((*str>='0'&&*str<='9')?(*str-'0'):(toupper(*str)-'A'+10))<<4;
      str++;
      *ptr++ |= ((*str>='0'&&*str<='9')?(*str-'0'):(toupper(*str)-'A'+10));
      str++;
    }
    else 
      *ptr++ = *str++;
  }
  *ptr = '\0';

  return result;
}


time_t
HttpScanDate (const char * date)
{
  static const char * month[12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
  };
  const char * ptr = date;
  struct tm ttm, gtm;
  int i, diff;
  time_t res, nres;

  while (isalpha(*ptr)) ptr++;
  while (isspace(*ptr) || *ptr==',') ptr++;

  if (atoi (ptr) != 0) {
    ttm.tm_mday = atoi (ptr);
    while (isdigit(*ptr)) ptr++;
    while (isspace(*ptr) || *ptr=='-') ptr++;
    for (i=0; i<12; i++) {
      if (fpstrnicmp (ptr, month[i], 3) == 0) {
	ttm.tm_mon = i;
	break;
      }
    }
    while (*ptr && !isspace(*ptr) && *ptr!='-') ptr++;
    while (isspace(*ptr) || *ptr=='-') ptr++;
    ttm.tm_year = atoi (ptr);
    while (isdigit(*ptr)) ptr++;
    while (isspace(*ptr) || *ptr=='-') ptr++;
  }
  else {
    for (i=0; i<12; i++) {
      if (fpstrnicmp (ptr, month[i], 3) == 0) {
	ttm.tm_mon = i;
	break;
      }
    }
    while (*ptr && !isspace(*ptr) && *ptr!='-') ptr++;
    ttm.tm_mday = atoi (ptr);
    while (isdigit(*ptr)) ptr++;
    while (isspace(*ptr) || *ptr=='-') ptr++;
  }

  ttm.tm_hour = atoi (ptr);
  while (isdigit(*ptr)) ptr++;
  if (*ptr == ':')      ptr++;
  ttm.tm_min  = atoi (ptr);
  while (isdigit(*ptr)) ptr++;
  if (*ptr == ':')      ptr++;
  ttm.tm_sec  = atoi (ptr);
  while (isdigit(*ptr)) ptr++;
  while (isspace(*ptr)) ptr++;

  if (isdigit(*ptr)) {
    ttm.tm_year = atoi (ptr);
  }

  if (ttm.tm_year >= 1900) {
    ttm.tm_year -= 1900;
  }

  ttm.tm_isdst  = 0;

  res = mktime (&ttm);

  /*
   * Korrigiere Zeitverschiebung.
   */

  fp_gmtime_r (&res, &gtm);

  gtm.tm_isdst = 0;
  nres = mktime (&gtm);

  res += res-nres;

  return res;
}

char *
HttpMakeDate (char *buf, int len, time_t now)
{
  struct tm gtm;

  if (!now) now = time (NULL);

  fp_gmtime_r (&now, &gtm);

  strftime (buf, len, "%a, %d %b %Y %H:%M:%S GMT",
	    &gtm);

  return buf;
}

/*
 * Ein paar reentrante Funktionen
 */

struct tm *
fp_localtime_r (const time_t *timer, struct tm *result)
{
#ifdef _REENTRANT
  static pthread_mutex_t mymutex = PTHREAD_MUTEX_INITIALIZER;

  pthread_mutex_lock (&mymutex);
#endif

  memcpy (result, localtime (timer), sizeof (struct tm));

#ifdef _REENTRANT
  pthread_mutex_unlock (&mymutex);
#endif

  return result;
}

struct tm *
fp_gmtime_r (const time_t *timer, struct tm *result)
{
#ifdef _REENTRANT
  static pthread_mutex_t mymutex = PTHREAD_MUTEX_INITIALIZER;

  pthread_mutex_lock (&mymutex);
#endif

  memcpy (result, gmtime (timer), sizeof (struct tm));

#ifdef _REENTRANT
  pthread_mutex_unlock (&mymutex);
#endif

  return result;
}
