/*
 * Uploading einer Datei mittels HTTP-Put
 */

#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <unistd.h>
#include <errno.h>
#include <address_class.h>
#include <socket_class.h>
#include "fpSocket.h"
#include "misc.h"

int
upload (fpSocket *sock, char *filename, char *url)
{
  char tempfile[1024], buf[1024];
  int total, count;
  struct stat st;
  int istemp;
  FILE *tf;

  if (filename[0] == '-' && !filename[1]) {
    tmpnam(tempfile);
    if ((tf = fopen (tempfile, "w+")) == NULL) {
      fprintf (stderr, "error: cannot open temp file %s\n", tempfile);
      return -1;
    }
    total = 0;
    while (!feof (stdin)) {
      count = fread (buf, 1, 1024, stdin);
      fwrite (buf, 1, count, tf);
      total += count;
    }
    fflush (tf);
    rewind (tf);
  }
  else {
    if (stat (filename, &st) != 0) {
      fprintf (stderr, "error: cannot stat %s\n", filename);
      return -1;
    }
    if ((tf = fopen (filename, "r")) == NULL) {
      fprintf (stderr, "error: cannot open %s\n", filename);
      return -1;
    }
    total = (long) st.st_size;
  }

  sock->printf ("PUT %s HTTP/1.1\r\n", url);
  sock->printf ("Content-Length: %d\r\n", total);
  sock->printf ("\r\n");

  while (total && !!*sock) {
    if ((count = fread (buf, 1, 1024, tf)) == 0) {
      break;
    }
    sock->write (buf, count);
    total -= count;
  }

  if (total) {
    fprintf (stderr, "error: premature EOF of input\n");
  }

  fclose (tf);

  if (istemp) {
    unlink (tempfile);
  }

  return (total) ? -1 : 0;
}

int
main (int argc, char *argv[])
{
  char hostname[1024], port[1024], url[1024], buf[1024];
  char *ptr, *p2;
  int res, count;

  if (argc != 3) {
    fprintf (stderr, "usage: %s <local-file> <remote-url>\n", argv[0]);
    exit (1);
  }

  ptr = argv[2];
  p2  = hostname;

  if (fpstrnicmp (ptr, "http://", 7) == 0) {
    ptr += 7;
  }

  while (*ptr && !isalnum (*ptr) && *ptr!='/' && *ptr!=':') ptr++;

  if (*ptr == ':' && *(ptr+1) == '/' && *(ptr+2) == '/') {
    fprintf (stderr, "error: can only handle http requests\n");
    exit (1);
  }

  if (isalnum(*ptr)) {
    while (*ptr && *ptr!=':' && *ptr!='/') {
      *p2++ = *ptr++;
    }
    *p2 = '\0';
  }
  else {
    strcpy (hostname, "localhost");
  }

  if (*ptr==':') {
    p2 = port;
    ptr++;
    while (*ptr && *ptr!='/') {
      *p2++ = *ptr++;
    }
    *p2 = '\0';
  }
  else {
    strcpy (port, "80");
  }

  if (*ptr == '/') {
    p2 = url;
    while (*ptr) {
      *p2++ = *ptr++;
    }
    *p2 = '\0';
  }
  else {
    strcpy (url, "/");
  }

  c_address addr(hostname, atoi(port));

  if (!addr) {
    fprintf (stderr, "illegal address: %s\n", strerror (errno));
    exit (1);
  }

  fpSocket sock;
  sock.connect (addr);

  if (!sock) {
    fprintf (stderr, "could not connect: %s\n", strerror (errno));
    exit (1);
  }

  res = upload (&sock, argv[1], url);

  if (res == 0) {
    while ((count = sock.read (buf, 1024)) != 0) {
      fwrite (buf, 1, count, stdout);
    }
  }

  sock.close();
}
